import os

import pytest
import requests
from nyckel import ImageClassificationFunction, OAuth2Renewer, TextClassificationFunction, TabularClassificationFunction


@pytest.fixture
def auth_test_user() -> OAuth2Renewer:
    user = get_test_user()
    assert_user_has_access(user)
    if not user_has_project(user):
        create_project_for_user(user)
    yield user


@pytest.fixture
def text_classification_function(auth_test_user):
    func = TextClassificationFunction.create_function("PYTHON-SDK TEXT TEST FUNCTION", auth_test_user)
    yield func
    func.delete()


@pytest.fixture
def image_classification_function(auth_test_user):
    func = ImageClassificationFunction.create_function("PYTHON-SDK IMAGE TEST FUNCTION", auth_test_user)
    yield func
    func.delete()


@pytest.fixture
def tabular_classification_function(auth_test_user):
    func = TabularClassificationFunction.create_function("PYTHON-SDK TABULAR TEST FUNCTION", auth_test_user)
    yield func
    # func.delete()


def get_test_user() -> OAuth2Renewer:
    if "NYCKEL_PYTHON_SDK_CLIENT_SECRET" in os.environ:
        auth = OAuth2Renewer(
            client_id="python-sdk-test",
            client_secret=os.environ["NYCKEL_PYTHON_SDK_CLIENT_SECRET"],
            server_url="https://www.nyckel-staging.com",
        )
    else:
        auth = OAuth2Renewer(
            client_id="python-sdk-test",
            client_secret="secret",
            server_url="http://localhost:5000",
        )
    return auth


def assert_user_has_access(user: OAuth2Renewer):
    try:
        user.token
    except:
        raise RuntimeError(f"Test client can not connect to {user.server_url}")


def user_has_project(user: OAuth2Renewer):
    session = requests.Session()
    session.headers.update({"authorization": "Bearer " + user.token})
    response = session.get(f"{user.server_url}/v0.9/projects")
    assert response.status_code == 200
    return len(response.json()) > 0


def create_project_for_user(user: OAuth2Renewer):
    session = requests.Session()
    session.headers.update({"authorization": "Bearer " + user.token})
    print("Creating project for user")
    response = session.post(f"{user.server_url}/v0.9/projects", json={"name": "my_project"})
    assert response.status_code == 200



from typing import Type

import fire
from nyckel import ImageClassificationFunction, OAuth2Renewer, TabularClassificationFunction, TextClassificationFunction
from nyckel.functions.classification.classification import ClassificationFunction
from nyckel.request_utils import get_session_that_retries


class NyckelFunctionDuplicator:
    def __init__(self, from_function_id: str, auth: OAuth2Renewer):
        self._from_function_id = from_function_id
        self._auth = auth
        self._session = get_session_that_retries()
        self._function_input_type: str

    def __call__(self) -> None:
        print(f"-> Starting copy process for {self._from_function_id} ...")
        self._refresh_auth_token()
        self._set_function_input_type()

        from_function = self._load_function()
        print(f"-> Loaded up function {self._from_function_id}")
        to_function = self._make_copy(from_function)
        if not to_function == from_function:
            print(f"-> Done!\n{from_function}\nsuccessfully copied to\n{to_function}")

    def _set_function_input_type(self) -> None:
        url = f"{self._auth.server_url}/v1/functions/{self._from_function_id}"
        response = self._session.get(url)
        assert response.status_code == 200, f"{response.text=} {response.status_code=}"
        if not response.json()["output"] == "Classification":
            raise ValueError("Can only copy classification functions.")
        function_input = response.json()["input"]
        assert function_input in ["Text", "Image", "Tabular"]
        self._function_input_type = function_input

    def _load_function(self) -> ClassificationFunction:
        return self._get_function_type()(self._from_function_id, self._auth)

    def _get_function_type(self) -> Type[ClassificationFunction]:
        if self._function_input_type == "Text":
            return TextClassificationFunction
        if self._function_input_type == "Image":
            return ImageClassificationFunction
        if self._function_input_type == "Tabular":
            return TabularClassificationFunction
        raise ValueError(f"Unknown input type {self._function_input_type=}")

    def _request_confirmation(self, function_name: str, function_id: str, n_labels: int, n_samples: int) -> bool:
        reply = input(
            f"Ok to proceed with copy of function: {function_name}, id: {function_id}, with {n_labels} labels and {n_samples} samples (y/n)? "
        )
        return reply == "y"

    def _make_copy(self, from_function: ClassificationFunction) -> ClassificationFunction:
        print(f"-> Reading labels and samples from {self._from_function_id}")
        labels = from_function.list_labels()
        samples = from_function.list_samples()

        if not self._request_confirmation(
            from_function.get_name(), from_function.function_id, len(labels), len(samples)
        ):
            print("Ok. Aborting...")
            return from_function

        new_function = self._get_function_type().create_function(f"COPY OF {from_function.get_name()}", self._auth)

        print(f"-> Adding labels and samples to {new_function.function_id}")
        new_function.create_labels(labels)
        new_function.create_samples(samples)
        return new_function

    def _refresh_auth_token(self) -> None:
        self._session.headers.update({"authorization": "Bearer " + self._auth.token})


def main(client_id: str, client_secret: str, from_function_id: str, server_url: str = "https://www.nyckel.com") -> None:
    auth = OAuth2Renewer(client_id, client_secret, server_url)
    duplicator = NyckelFunctionDuplicator(from_function_id, auth)
    duplicator()


if __name__ == "__main__":
    fire.Fire(main)
